VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsAPI_AI"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' // clsAPI_AI.cls - class for speech recognition using API.AI
' //  Krivous Anatoly Anatolevich (The trick), 2016

Option Explicit

Private Declare Sub CopyMemory Lib "kernel32" _
                    Alias "RtlMoveMemory" ( _
                    ByRef Destination As Any, _
                    ByRef Source As Any, _
                    ByVal Length As Long)

Private mAccessToken    As String
Private mVersion        As String
Private mLanguage       As String
Private request         As XMLHTTP

Public Event Answer(ByVal text As String)

Sub OnReadyStateChange()
Attribute OnReadyStateChange.VB_UserMemId = 0
Attribute OnReadyStateChange.VB_MemberFlags = "40"
    Dim json    As clsJSONParser
    Dim var     As Variant
    
    On Error GoTo error_handler
    
    If request.readyState = 4 Then
        
        Set json = New clsJSONParser
        
        If Not json.Parse(request.responseText) Then
            MsgBox "Error response"
            Exit Sub
        End If
        
        var = json.TreeItemFromPath("result\resolvedQuery")
        
        If IsEmpty(var) Then
            Dim code            As Long
            Dim error_type      As String
            Dim error_details   As String
            
            code = json.TreeItemFromPath("status\code")
            error_type = json.TreeItemFromPath("status\errorType")
            error_details = json.TreeItemFromPath("status\errorDetails")
            
            MsgBox "Error " & Str(code) & vbNewLine & error_type & vbNewLine & error_details, vbCritical
            
        Else
            RaiseEvent Answer(var)
        End If
        
    End If
    
    Exit Sub
    
error_handler:
    
    MsgBox "An error has occured"
    
End Sub

Public Property Let Language( _
                    ByRef Value As String)
    mLanguage = Value
End Property
Public Property Get Language() As String
    Language = mLanguage
End Property

Public Property Let AccessToken( _
                    ByRef Value As String)
    mAccessToken = Value
End Property
Public Property Get AccessToken() As String
    AccessToken = mAccessToken
End Property

Public Property Let Version( _
                    ByRef Value As String)
    mVersion = Value
End Property
Public Property Get Version() As String
    Version = mVersion
End Property

Public Function SpeechToText( _
                ByRef rawData() As Byte) As String
    Dim query       As String:  Dim preBody     As String
    Dim postBody    As String:  Dim binBody()   As Byte
    Dim boundary    As String:  Dim index       As Long
    
    If request Is Nothing Then
        Set request = New XMLHTTP
    End If
    
    If request.readyState <> 0 And request.readyState <> 4 Then
        MsgBox "Wait for previous response!"
        Exit Function
    End If
    
    query = "https://api.api.ai/v1/query?v=" & mVersion
    
    boundary = "--------" & GenerateRandomString()
    
    preBody = StrConv("--" & boundary & vbNewLine & "Content-Disposition: form-data; name=" & _
           Chr$(34) & "request" & Chr$(34) & vbNewLine & "Content-Type: application/json" & _
           vbNewLine & vbNewLine & "{'lang':'" & mLanguage & "', 'sessionId':'" & _
           GenerateRandomString() & "'}" & vbNewLine & "--" & boundary & vbNewLine & _
           "Content-Disposition: form-data; name=" & Chr$(34) & "voiceData" & Chr$(34) & _
           vbNewLine & "Content-Type: audio/wav" & vbNewLine & vbNewLine, vbFromUnicode)
           
    postBody = StrConv(vbNewLine & "--" & boundary & "--" & vbNewLine, vbFromUnicode)
    
    ReDim binBody(LenB(preBody) + (UBound(rawData) + 1) + LenB(postBody) - 1)
    
    CopyMemory binBody(index), ByVal StrPtr(preBody), LenB(preBody):    index = index + LenB(preBody)
    CopyMemory binBody(index), rawData(0), UBound(rawData) + 1:         index = index + UBound(rawData) + 1
    CopyMemory binBody(index), ByVal StrPtr(postBody), LenB(postBody):  index = index + LenB(postBody)
    
    request.open "POST", query
    request.setRequestHeader "Authorization", "Bearer " & mAccessToken
    request.setRequestHeader "Content-Type", "multipart/form-data; boundary=" & boundary
    
    request.OnReadyStateChange = Me
    
    ' // Send body
    request.send binBody()

End Function

Private Function GenerateRandomString() As String
    Dim i       As Long
    Dim count   As Long
    
    count = (Rnd() * 26) + 10
    
    For i = 0 To count
        GenerateRandomString = GenerateRandomString & Chr$(Asc("0") + Int(Rnd() * 10))
    Next
    
End Function

Private Sub Class_Initialize()
    mLanguage = "en"
End Sub
