VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsJSONParser"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' // clsJSONParser.cls - simple JSON parser
' //  Krivous Anatoly Anatolevich (The trick), 2016

Option Explicit

Public Enum JSONStructure
    JS_OBJECT
    JS_ARRAY
    JS_STRING
    JS_NUMBER
    JS_KEYWORD
End Enum

Private mTree As Collection

Public Property Get Tree() As Collection
    Set Tree = mTree
End Property

Public Property Get TreeItemFromPath( _
                    ByRef strPath As String) As Variant
    Dim items   As Variant
    Dim var     As Variant
    Dim index   As Variant
    Dim col     As Collection
    Dim subItem As Collection
    
    items = Split(strPath, "\")
    Set col = mTree
    
    For Each var In items
        
        If col.count = 0 Then Exit Property
        
        If col(1) = JS_OBJECT Then
            
            Set subItem = col(2)
            
            For index = 2 To col.count
                
                If StrComp(col(index)(1), var, vbTextCompare) = 0 Then
                
                    Set col = col(index)(2)
                    GoTo continue
                    
                End If
                    
            Next
        
        End If
              
        Exit Property
        
continue:
        
    Next
    
    If IsObject(col(2)) Then
        Set TreeItemFromPath = col(2)
    Else
        TreeItemFromPath = col(2)
    End If
    
End Property

Public Property Get ItemProperty( _
                    ByVal Obj As Collection, _
                    ByRef PropName As String) As Variant
    Dim itm As Variant: Dim i   As Long
    
    If Obj Is Nothing Then Exit Property
    If Obj.count = 0 Then Exit Property
    If Obj(1) <> JS_OBJECT Then i = 1
    
    For Each itm In Obj
    
        If i Then
            If StrComp(itm(1), PropName, vbTextCompare) = 0 Then Set ItemProperty = itm: Exit Property
        End If
        
        i = i + 1
        
    Next
    
End Property

Public Function Parse( _
                ByRef Str As String) As Boolean
    Dim ret     As Variant: Dim l       As Long
    Dim status  As Boolean
    
    Set ret = ExtractValue(Str, 1, l, Parse)
    
    If Parse Then Set mTree = ret
    
End Function

Private Function ExtractKeyValuePair( _
                 ByRef Str As String, _
                 ByVal Start As Long, _
                 ByRef Length As Long) As Collection
    Dim Key     As String:      Dim Value   As Variant
    Dim ret     As Collection:  Dim l       As Long
    Dim status  As Boolean:     Dim init    As Long
    
    Set ret = New Collection
    
    init = Start
    
    Start = SkipSpace(Str, Start)
    Key = ExtractString(Str, Start, status)
    If Not status Then Exit Function
    Start = Start + Len(Key) + 2
    
    Start = SkipSpace(Str, Start)
    If Mid$(Str, Start, 1) <> ":" Then Exit Function
    Start = Start + 1
    Start = SkipSpace(Str, Start)
    
    ret.Add Key
    Set Value = ExtractValue(Str, Start, l, status)
    If Not status Then Exit Function
    ret.Add Value
    
    Set ExtractKeyValuePair = ret
    Length = Start - init + l
    
End Function

Private Function ExtractValue( _
                 ByRef Str As String, _
                 ByVal Start As Long, _
                 ByRef Lenght As Long, _
                 ByRef status As Boolean) As Collection
    Dim Value   As Collection:  Dim stat    As Boolean
    
    status = False
    
    Select Case Mid$(Str, Start, 1)
    Case Chr$(34)
        Dim strValue As String
        
        strValue = ExtractString(Str, Start, stat)
        If Not stat Then Exit Function
        
        Set Value = New Collection
        Value.Add JS_STRING
        Value.Add strValue
        
        Lenght = Len(strValue) + 2
        
    Case "0" To "9", "-"
        Dim numValue As String
        
        numValue = ExtractNumber(Str, Start, stat)
        If Not stat Then Exit Function
        
        Set Value = New Collection
        Value.Add JS_NUMBER
        Value.Add numValue
        
        Lenght = Len(numValue)
        
    Case "t", "f", "n"
        Dim keyValue As String
        
        keyValue = ExtractKeyword(Str, Start, stat)
        If Not stat Then Exit Function
        
        Set Value = New Collection
        Value.Add JS_KEYWORD
        Value.Add keyValue
        
        Lenght = Len(keyValue)
    Case "{"

        Set Value = ExtractObject(Str, Start, Lenght, stat)
        If Not stat Then Exit Function
    Case "["
        Dim arrValue As Collection
        
        Set Value = ExtractArray(Str, Start, Lenght, stat)
        If Not stat Then Exit Function
        
    Case Else: Exit Function
    End Select
    
    Set ExtractValue = Value

    status = True
    
End Function


Private Function ExtractKeyword( _
                 ByRef Str As String, _
                 ByVal Start As Long, _
                 ByRef status As Boolean) As String
                 
    status = False
    
    If Mid$(Str, Start, 4) = "true" Then
        ExtractKeyword = "true"
    ElseIf Mid$(Str, Start, 5) = "false" Then
        ExtractKeyword = "false"
    ElseIf Mid$(Str, Start, 4) = "null" Then
        ExtractKeyword = "null"
    Else: Exit Function
    End If
    
    status = True
    
End Function

Private Function ExtractObject( _
                 ByRef Str As String, _
                 ByVal Start As Long, _
                 ByRef Length As Long, _
                 ByRef status As Boolean) As Collection
    Dim i       As Long:        Dim Obj     As String
    Dim Value   As Variant:     Dim l       As Long
    Dim ret     As Collection

    status = False
    Obj = ExtractBlock(Str, Start)
    If Len(Obj) = 0 Then Exit Function
    Start = SkipSpace(Obj, Start)
    
    i = 1
    Set ret = New Collection
    ret.Add JS_OBJECT
    
    Do
        Set Value = ExtractKeyValuePair(Obj, i, l)
        ret.Add Value
        i = i + l
        i = SkipSpace(Obj, i)
        If i > Len(Obj) Then Exit Do
        If Mid$(Obj, i, 1) <> "," Then Exit Function
        i = i + 1
    Loop While i <= Len(Obj)
    
    Set ExtractObject = ret
    Length = i + 1
    
    status = True
    
End Function

Private Function ExtractArray( _
                 ByRef Str As String, _
                 ByVal Start As Long, _
                 ByRef Length As Long, _
                 ByRef status As Boolean) As Collection
    Dim i       As Long:        Dim Arr     As String
    Dim Value   As Collection:  Dim l       As Long
    Dim stat    As Boolean:     Dim begin   As Long
    Dim ret     As Collection
    
    status = False
    
    Arr = ExtractBlock(Str, Start)
    
    Start = SkipSpace(Str, Start)
    
    i = 1
    
    Set ret = New Collection
    
    ret.Add JS_ARRAY
    
    If Len(Arr) = 0 Then
        status = True
        Length = 2
        Set ExtractArray = ret
        Exit Function
    End If
    
    Do
        Set Value = ExtractValue(Arr, i, l, stat)
        ret.Add Value
        If Not stat Then Exit Function
        i = i + l
        i = SkipSpace(Arr, i)
        If i > Len(Arr) Then Exit Do
        If Mid$(Arr, i, 1) <> "," Then Exit Function
        i = i + 1
    Loop While i <= Len(Arr)
    
    Set ExtractArray = ret
    Length = i + 1
    
    status = True
    
End Function

Private Function ExtractString( _
                 ByRef Str As String, _
                 ByVal Start As Long, _
                 ByRef status As Boolean) As String
    Dim i As Long
    
    status = False
    
    If Mid$(Str, Start, 1) <> Chr$(34) Then Exit Function
    
    i = Start
    
    Do
        i = InStr(i + 1, Str, Chr$(34))
        If i = 0 Then Exit Function
    Loop While Mid$(Str, i - 1, 1) = "\"
    
    ExtractString = Mid$(Str, Start + 1, i - Start - 1)
    status = True
    
End Function

Private Function ExtractNumber( _
                 ByRef Str As String, _
                 ByVal Start As Long, _
                 ByRef status As Boolean) As String
    Dim point   As Boolean: Dim begin   As Long
    
    begin = Start
    status = False
    
    Do
        Select Case Mid$(Str, Start, 1)
        Case "-": If Start > begin Then Exit Function
        Case ".": If point Then Exit Function Else point = True
        Case "0" To "9"
        Case Else: Exit Do
        End Select
        Start = Start + 1
    Loop While Start <= Len(Str)
    
    ExtractNumber = Mid$(Str, begin, Start - begin)
    status = True
    
End Function

Private Function SkipSpace( _
                 ByRef Str As String, _
                 ByVal Start As Long) As Long
                 
    Do
    
        Select Case Mid$(Str, Start, 1)
        Case " ", vbCr, vbLf, vbTab, vbBack
        Case Else: Exit Do
        End Select
        Start = Start + 1
        
    Loop While Start <= Len(Str)
    
    SkipSpace = Start
    
End Function

Public Function ExtractBlock( _
                ByRef Str As String, _
                ByVal Start As Long) As String
    Dim iFind   As String:  Dim dFind   As String
    Dim c       As Long:    Dim i       As Long

    iFind = Mid$(Str, Start, 1)
    Select Case iFind
    Case "{": dFind = "}"
    Case "[": dFind = "]"
    Case Else: Exit Function
    End Select
    
    c = 1
    Start = Start + 1
    i = Start
    
    Do While i <= Len(Str) And c > 0
    
        Select Case Mid$(Str, i, 1)
        Case iFind: c = c + 1
        Case dFind: c = c - 1
        End Select
        i = i + 1
        
    Loop
    
    If c = 0 Then ExtractBlock = Mid$(Str, Start, i - Start - 1)

End Function
